<?php

/**
 * Plugin kzDemo
 *
 * Crée un mode démo pour le back-office.
 * @author	Jean-Pierre Pourrez "bazooka07"
 * */
if(!defined('PLX_ROOT')) {
	die('Bye !');
}

class kzDemo extends plxPlugin {
	const PARAM_NAME = 'profil';
	const BEGIN_CODE = '<?php' . PHP_EOL;
	const END_CODE = PHP_EOL . '?>' . PHP_EOL;
	const HOOKS_BASE = Array(
		'plxMotorConstruct'			=> 'plxMotorConstruct',

		'AdminPrepend'				=> 'AdminPrepend',
		'AdminAuth'					=> 'AdminAuth',
		'AdminAuthEndHead'			=> 'AdminAuthEndHead',
		'AdminAuthEndBody'			=> 'AdminAuthEndBody',
	);
	const HOOKS_USER_DEMO = Array(
		'AdminTopBottom'			=> 'AdminTopBottom',
		'plxAdminEditUsersBegin'	=> 'plxAdminEditUsersBegin',
		'plxAdminEditArticle'		=> 'checkUserDemo',
		'plxMotorPreChauffageBegin'	=> 'plxMotorPreChauffageBegin',
		'plxMotorDemarrageEnd'		=> 'plxMotorDemarrageEnd',
		'plxMotorNewCommentaire'	=> 'plxMotorNewCommentaire',
		'AdminArticleInitData'		=> 'AdminArticleInitData',
	);
	const LOGINS = array(
		# use of plxUtils::charAleatoire(16)
		PROFIL_ADMIN		=> 'rl4jNee09OQszXyz',
		PROFIL_MANAGER		=> '1QxKN6FJWg72f2kg',
		PROFIL_MODERATOR	=> '26QRN9ccltLhL6hR',
		PROFIL_EDITOR		=> 'JxSKT3ZXHaUpunJX',
		PROFIL_WRITER		=> 'PO9HZIB8O49gXXqN',
	);
	const USER_ID = 950;
	const MAIL_DOMAIN = '@no.com';
	private $_demo = false;

	public function __construct($default_lang) {
		parent::__construct($default_lang);
		if(defined('PLX_ADMIN')) {
			parent::setConfigProfil(PROFIL_ADMIN);
		}

		$this->_demo = (isset($_SESSION['user']) and intval($_SESSION['user']) >= self::USER_ID);

		foreach(self::HOOKS_BASE as $k=>$v) {
			$this->addHook($k, $v);
		}

		if($this->_demo) {
			foreach(self::HOOKS_USER_DEMO as $k=>$v) {
				$this->addHook($k, $v);
			}
		}
	}

	public function getProfils($min=false) {
		# cf core/admin/parametres_users.php
		$profils = array(
			PROFIL_ADMIN		=> L_PROFIL_ADMIN,
			PROFIL_MANAGER		=> L_PROFIL_MANAGER,
			PROFIL_MODERATOR	=> L_PROFIL_MODERATOR,
			PROFIL_EDITOR		=> L_PROFIL_EDITOR,
			PROFIL_WRITER		=> L_PROFIL_WRITER
		);

		if(is_numeric($min)) {
			$min = intval($min);
		}
		return is_integer($min) ? array_filter($profils, function($a) use($min) { return ($a >= $min); },  ARRAY_FILTER_USE_KEY) : $profils;
	}

	private function _getLogin($profil) {
		$str = $_SESSION[__CLASS__] . self::LOGINS[$profil];
		return strtoupper(base_convert(crc32($str), 10, 36));
	}

	public function getLogins() {
		return array(
			PROFIL_ADMIN		=> $this->_getLogin(PROFIL_ADMIN),
			PROFIL_MANAGER		=> $this->_getLogin(PROFIL_MANAGER),
			PROFIL_MODERATOR	=> $this->_getLogin(PROFIL_MODERATOR),
			PROFIL_EDITOR		=> $this->_getLogin(PROFIL_EDITOR),
			PROFIL_WRITER		=> $this->_getLogin(PROFIL_WRITER),
		);
	}

	/* ==== HOOKS ==== */

	/**
	 * Add users for demonstration
	 * */
	public function plxMotorConstruct() {
		$addUsers = true;
		if(defined('PLX_AUTHPAGE')) {
			if(
				(!empty($_POST) and !isset($_POST['password'])) or
				(isset($_GET['action']) and $_GET['action'] == 'lostpassword')
			) {
				$addUsers = false;
			} else if(!isset($_SESSION[__CLASS__])) {
				# Make a random password. Required for authentication
				$_SESSION[__CLASS__] = plxUtils::charAleatoire(16);
			}
		} elseif(
			!isset($_SESSION['user']) or
			!isset($_SESSION['profil']) or
			$_SESSION['user'] < self::USER_ID
		) {
			if(isset($_SESSION[__CLASS__])) {
				#Drop the random passsword
				unset($_SESSION[__CLASS__]);
			}
			$addUsers = false;
		}

		if($addUsers) {
			$name = $this->getLang('DEMO_USER');
			$userInfos = $this->getLang('DEMO_USER_INFO');
			$mailPattern = strtolower(__CLASS__) . '-%s' . self::MAIL_DOMAIN;
			echo self::BEGIN_CODE;
?>
# hide the real email for each user
foreach(array_keys($this->aUsers) as $id) {
	$this->aUsers[$id]['email'] = sprintf('<?= $mailPattern?>', $id);
}

$kzDemo = $this->plxPlugins->aPlugins['<?= __CLASS__ ?>'];
foreach($kzDemo->getLogins() as $profil=>$login) {
	$id = <?= self::USER_ID ?> + $profil;
	$user_id = str_pad($id, 3, 0, STR_PAD_LEFT);
	$salt = plxUtils::charAleatoire(10);
	$encryptPassword = sha1($salt . md5($_SESSION['<?= __CLASS__ ?>']));
	$this->aUsers[$user_id] = array(
		'active'	=> 1,
		'delete'	=> 0,
		'profil'	=> $profil,
		'login'		=> $login,
		'name'		=> sprintf('<?= $name ?>', $profil + 1),
		'password'	=> $encryptPassword,
		'salt'		=> $salt,
		'infos'		=> '<?= $userInfos ?>',
		'email'		=> sprintf('<?= $mailPattern?>', $user_id),
		'lang'		=> $this->aConf['default_lang'],
		'password_token'		=> '',
		'password_token_expiry'	=> '',
	);
}
<?php
			echo self::END_CODE;
		}
	}

	/**
	 * Bloque les formulaires du back-office
	 **/
	public function AdminPrepend() {
		if(!empty($_SESSION['preview']) and basename($_SERVER['SCRIPT_NAME'], '.php') != 'article') {
			unset($_SESSION['preview']);
		}

		if(defined('PLX_AUTHPAGE') or intval($_SESSION['user']) < self::USER_ID) {
			return;
		}

		if(!empty($_POST)) {
			$referer = parse_url($_SERVER['HTTP_REFERER']);
			$script = basename($referer['path'], '.php');
			if(substr($referer['path'], -1) == '/') {
				$script = 'index';
			}
			if(
				($script == 'index' and isset($_POST['sel_cat']) and preg_match('#^\d{3}$#', $_POST['sel_cat'])) or
				($script == 'article' and isset($_POST['preview']))  or
				($script == 'medias' and isset($_POST['btn_changefolder'])) or
				($script == 'parametres_edittpl' and isset($_POST['load']) and isset($_POST['template']) and preg_match('#^\w[\w/-]*\.(?:php|js|css)$#', $_POST['template']))
			) {
				return;
			}

			plxMsg::Error($this->getLang('SAVE_DISABLED'));
			header('Location: ' . $_SERVER['HTTP_REFERER']);
			exit;
		}

		if(isset($_GET['del']) and $_GET['del'] == 'install') {
			# Suppression des fichiers d'installation'
			plxMsg::Error($this->getLang('SAVE_DISABLED'));
			header('Location: ' . $_SERVER['HTTP_REFERER']);
			exit;
		}
	}

	/**
	 * Panneau d'accueil dans la page d'authentification pour la démonstration
	 **/
	 public function AdminAuth() {
?>
	<div>
		<div>
			<?php $this->lang('AUTH_GREETING'); ?> :
		</div>
		<ul id="kzdemoLogins">
<?php
	$min = $this->getParam(self::PARAM_NAME);
	$min = is_numeric($min) ? intval($min) : PROFIL_ADMIN;
	$aProfils = $this->getProfils();
	foreach($this->getLogins() as $profil=>$login) {
		if($profil >= $min) {
?>
			<li data-login="<?= $login ?>" title="Login: <?= $login ?>"><?= $aProfils[$profil] ?></li>
<?php
		}
	}
?>
		</ul>
	</div>
<?php
	}

	/**
	 * Ajout régles CSS pour la page d'authentification
	 **/
	public function AdminAuthEndHead() {
?>
	<style>
		#kzdemoLogins li {
			padding: 0.25rem 1rem;
			cursor: pointer;
		}
		#kzdemoLogins li:hover {
			background-color: #258fd6;
			color: #eee;
		}
 	</style>
<?php
	}

	/*
	 * Auto-remplit le champ password dans le formulaire d'authentification
	 * */
	public function AdminAuthEndBody() {
?>
	<script>
	(function() {
		'use strict';
		const container = document.getElementById('kzdemoLogins');
		if(container != null) {
			container.addEventListener('click', function (event) {
				if(event.target.dataset.login != null) {
					event.preventDefault();
					var myForm = document.forms[0];
					myForm.login.value = event.target.dataset.login;
					myForm.password.value = '<?= $_SESSION[__CLASS__] ?>';
				}
			});
		}
	})();
	</script>
<?php
	}

	/**
	 * Displays a warning for each admin page
	 **/
	public function AdminTopBottom() {
		if(intval($_SESSION['user']) < self::USER_ID) {
			return;
		}
?>
	<div class="in-action-bar <?= __CLASS__ ?>"><?php $this->lang('DEMO_MODE'); ?></div>
<?php
	}

	/**
	 * Supprime tous les utilisateurs créés pour la démo
	 **/
	public function plxAdminEditUsersBegin() {
		echo self::BEGIN_CODE;
?>
# Sauvegarde désactivée pour la démo
plxMsg::Error('<?= $this->getLang('SAVE_DISABLED') ?>');
return true;
<?php
		echo self::END_CODE;
	}

	public function checkUserDemo() {
		echo self::BEGIN_CODE;
?>
if(intval($_SESSION['user']) >= <?= self::USER_ID ?>) {
	plxMsg::Error('<?= $this->getLang('SAVE_DISABLED') ?>');
	return true;
}
<?php
		echo self::END_CODE;
	}

	public function plxMotorPreChauffageBegin() {
		if(empty($_SESSION['user']) or intval($_SESSION['user']) < self::USER_ID) {
			return;
		}
		echo self::BEGIN_CODE;
?>
# <?= __CLASS__ ?> plugin
if(
	!empty($_SESSION['preview']) and
	!empty($this->get) and
	preg_match('#^article(\d+)\b#', $this->get, $matches) and
	str_pad($matches[1], 4, '0', STR_PAD_LEFT) == $_SESSION['preview'][0]['numero']
) {
	$this->mode = 'preview';
	return true;
}
<?php
		echo self::END_CODE;
	}

	public function plxMotorDemarrageEnd() {
		$message = $this->getLang('COMMENT_NOT_SAVED');
		echo self::BEGIN_CODE;
?>
# <?= __CLASS__ ?> plugin
if(
	!empty($_SESSION['preview']) and
	$this->mode == 'article' and
	$this->aConf['allow_com'] and
	$this->plxRecord_arts->f('allow_com') and
	!empty($_POST['content'])
) {
	$content = plxUtils::unSlash($_POST);
	#  Si nécessaire, on vérifie que le capcha est correct
	if(
		!empty($this->aConf['capcha']) AND (
			empty($_SESSION['capcha_token']) OR
			empty($_POST['capcha_token']) or
			$_SESSION['capcha_token'] != $_POST['capcha_token'] or
			$_SESSION['capcha'] != sha1($content['rep'])
		)
	) {
		$kzMsg = L_NEWCOMMENT_ERR_ANTISPAM;
	} elseif (empty(trim($content['name'])) or empty(trim($content['content']))) {
		# Certains champs obligatoires sont vides
		$kzMsg = L_NEWCOMMENT_FIELDS_REQUIRED;
	} else {
		# Commentaire non enregistré pendant la démonstration
		$kzMsg = '<?= $message ?>';
	}

	$_SESSION['msgcom'] = $kzMsg;
	$_SESSION['msg'] = array();
	foreach(array(
		'name',
		'site',
		'mail',
		'content',
		'parent',
	) as  $k) {
		$_SESSION['msg'][$k] = $content[$k];
	}
}
<?php
		echo self::END_CODE;
	}

	public function plxMotorNewCommentaire() {
		$message = $this->getLang('COMMENT_NOT_SAVED');
		echo self::BEGIN_CODE;
?>
return '<?= $message ?>';
<?php
		echo self::END_CODE;
	}

	public function AdminArticleInitData() {
		$title = plxUtils::strRevCheck($this->getLang('NEW_ARTICLE_TITLE'));
		echo self::BEGIN_CODE;
?>
$title = '<?= $title ?>';
<?php
		echo self::END_CODE;
	}

}
